#include <sys/systm.h>
#include <sys/uio.h>
#include <sys/conf.h>
#include <miscfs/devfs/devfs.h>
#include <mach/mach_types.h>
#include <IOKit/IOLib.h>


#define BUF_SIZE 1024
static char *state;

IOLock *driver_mtx;


static int 
cdev_test_open(dev_t dev, int flags, int devtype, struct proc *p)
{
	char *s;
	int err;

	if (state != NULL)
		return EBUSY;
	
	s = IOMalloc(BUF_SIZE);
	if (s == NULL) {
		IOLog("No memory!\n");
		return ENOMEM;
	}
	IOLockLock(driver_mtx);
	if (state == NULL) {
		err = 0;
		state = s;
	} else {
		err = EBUSY;
	}
	IOLockUnlock(driver_mtx);
	return (err);
}

static int 
cdev_test_close(dev_t dev, int flags, int devtype, struct proc *p)
{
	char *s;

	s = state;
	IOLockLock(driver_mtx);
	state = NULL;
	IOLockUnlock(driver_mtx);
	if (s != NULL)
		IOFree(s, BUF_SIZE);
	return (0);
}



static struct cdevsw cdev_test_cdevsw = {
	(d_open_t *)&cdev_test_open,
	(d_close_t *)&cdev_test_close,
	eno_rdwrt,
	eno_rdwrt,
	eno_ioctl,
	eno_stop,
	eno_reset,
	0,
	eno_select,
	eno_mmap,
	eno_strat,
	eno_getc,
	eno_putc,
	0,
};


int cdev_test_major = -1;
void *cdev_test_node = NULL;

kern_return_t
cdev_test_start(kmod_info_t * ki, void *data)
{

	driver_mtx = IOLockAlloc();
	cdev_test_major = cdevsw_add(-1, &cdev_test_cdevsw);
	if (cdev_test_major == -1) {
		printf("cdevsw_add failed!\n");
		IOLockFree(driver_mtx);
		return KERN_FAILURE;
	}

	cdev_test_node = devfs_make_node(makedev(cdev_test_major, 0),
				     DEVFS_CHAR, UID_ROOT, GID_WHEEL,
				     0666, "cdev_test");

	if (cdev_test_node == NULL) {
		printf("devfs_make_node failed!\n");
		(void)cdevsw_remove(cdev_test_major, &cdev_test_cdevsw);
		IOLockFree(driver_mtx);
		return KERN_FAILURE;

	}
	printf("cdev_test loaded\n");


	return KERN_SUCCESS;
}
kern_return_t
cdev_test_stop(kmod_info_t * ki, void *data)
{

	devfs_remove(cdev_test_node);
	(void)cdevsw_remove(cdev_test_major, &cdev_test_cdevsw);
	IOLockFree(driver_mtx);
	printf("cdev_test detached, state = %p!\n", state);
	return KERN_SUCCESS;
}

KMOD_EXPLICIT_DECL (com.none.kext.cdev_test, "0.0.0", cdev_test_start, cdev_test_stop)

